searchData={"items":[{"type":"module","title":"Komodo","doc":"","ref":"Komodo.html"},{"type":"module","title":"Komodo.Components","doc":"","ref":"Komodo.Components.html"},{"type":"function","title":"Komodo.Components.js_component/1","doc":"Function component for rendering javascript components\n\nIt uses a \"props-in, callbacks-out\" data-flow, so for example a React component that is used in React like so:\n\n```js\n \n```\n\nwould be rendered from a live view as\n```heex\n<.js_component name=\"MyComponent\" props={%{count: 43}} callbacks={%{onIncrement: \"increment\"}} />\n```\nwhere `\"increment\"` is the event name sent back to the live view.","ref":"Komodo.Components.html#js_component/1"},{"type":"function","title":"Attributes - Komodo.Components.js_component/1","doc":"* `tag_name` (`:atom`) - Defaults to `:div`.\n* `id` (`:string`) (required)\n* `name` (`:string`) (required)\n* `props` (`:map`) - Defaults to `%{}`.\n* `callbacks` (`:map`) - Defaults to `%{}`.\n* `class` (`:string`) - Defaults to `\"\"`.\n* `style` (`:string`) - Defaults to `\"\"`.","ref":"Komodo.Components.html#js_component/1-attributes"},{"type":"module","title":"Komodo.Helpers","doc":"","ref":"Komodo.Helpers.html"},{"type":"function","title":"Komodo.Helpers.arguments_path/1","doc":"Converts a string path spec to an array for use in the komodo javascript library\n\nExample:\niex> Komodo.Helpers.arguments_path(\"&1.inner[2].name\")\n\"0.inner.2.name\"","ref":"Komodo.Helpers.html#arguments_path/1"},{"type":"function","title":"Komodo.Helpers.normalise_callback_spec/1","doc":"","ref":"Komodo.Helpers.html#normalise_callback_spec/1"},{"type":"extras","title":"Komodo","doc":"# Komodo\n\nUse components from Javascript frameworks in your [Phoenix Liveview](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html) with ease!\n\n![Demo](assets/komodo.gif)\n\nNOTE: this library does **not** cover bundling/transpiling of components for each Javascript framework - it's concerned more with providing a standard interface for using the components in LiveView. However each README for supported frameworks will give pointers where necessary which should be enough to get it working.","ref":"readme.html"},{"type":"extras","title":"Background - Komodo","doc":"Many Javascript frameworks allow for reusing components by employing a **props-in** (i.e. data), **callbacks-out** (or \"events-out\") model. Komodo allows for using these components in your Phoenix LiveView in the same way.\n\nFor example, if you were to use a React component like so:\n\n```jsx\n \n```\n\nthen you could use this component from your LiveView like so:\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n  <.js_component\n    id=\"my-react-component\"\n    name=\"MyReactComponent\"\n    props={%{\n      user: @user\n    }}\n    callbacks={%{\n      onChangeUser: {\"update_user\", \"&1\"}\n    }}\n  />\n  \"\"\"\nend\n\ndef handle_event(\"update_user\", new_user, socket) do\n  // ...\nend\n```\n\nThis assumes a Javascript component with the name `\"MyReactComponent\"` has been registered (see [Setup](#setup) below), using an adapter for the specific framework (in this case React).\n\nAdapters are small pieces of Javascript code that wrap a framework component into a standardized JS component that is compatible with the code above.\n\nSee [Supported Adapters](#supported-adapters) below for supported frameworks and how to create adapters for other frameworks.\n\nTo understand how parameters are sent from callbacks to `handle_event`, see [Callback Parameters](#callback-parameters) below.","ref":"readme.html#background"},{"type":"extras","title":"Installation - Komodo","doc":"This package is [on Hex](https://hexdocs.pm/komodo), so you can add`komodo` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:komodo, \"~> 0.1.1\"}\n  ]\nend\n```\n\nand run `mix deps.get`.\n\nAdd the javascript library to your assets `package.json`:\n\n```js\n\"dependencies\": {\n  // ...\n  \"komodo\": \"file:../deps/komodo\"\n}\n```\n\nand install - `npm install --prefix assets`.","ref":"readme.html#installation"},{"type":"extras","title":"Setup - Komodo","doc":"Given a Phoenix app `MyApp`:\n\n1. Import the provided components for use in heex templates\n\nIn `my_app_web.ex`:\n\n```diff\ndefmodule MyAppWeb do\n\n  # ...\n\n  def html_helpers do\n    # ...\n+    import Komodo.Components\n    # ...\n  end\n\n  # ...\nend\n```\n\n2. Add the provided hook to the LiveSocket\n\nIn `app.js`:\n\n```diff\n// ...\n+ import { registerJsComponents } from \"komodo\";\n\n// ...\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  // ...\n  hooks: {\n+    komodo: registerJsComponents({\n+      // individual JS components will go here\n+    })\n  }\n});\n\n// ...\n```","ref":"readme.html#setup"},{"type":"extras","title":"Callback parameters - Komodo","doc":"A string specification is used to tell `js_component` how to transform callback payloads into a `parameters` argument to send back to the liveview.\nIt makes use of the `&` to specify the argument number (which should already be familiar to Elixir users), as well as `.` and `[]` to access nested data.\n\nFor example, `\"&1\"` means the first argument, and `\"&1.users[2]\"` means the 3rd item in the users property of the first argument.\n\nThis should be intuitive as it follows Elixir/Javascript convention (note that the argument number `&1` starts from 1 whereas the array access index `[2]` starts from 0).\n\nThis will be better understood with an example...\n\nSupposing the javascript component has a callback `onChangeTrack` that emits two arguments:\n\n- the new song `{title: \"\"El Pocito\"}`\n- the artist `{name: \"Vicente Amigo\"}`\n\ni.e. in javascript you would do something like\n\n```jsx\nonChangeTrack={(newSong, newArtist) => handleChangeTrack(...)}\n```\n\nThen the liveview will have\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n  <.js_component\n    ...\n    callbacks={%{\n      onChangeTrack: ???\n    }}\n  />\n  \"\"\"\nend\n\ndef handle_event(\"change_track\", parameters, socket) do\n  // ...\nend\n```\n\nThe table below shows what to put in place of `???` to form the `parameters` argument given to `handle_event`.\n\n| Callback spec (`???`)                                      | `parameters`                                   | Notes                                       |\n| ---------------------------------------------------------- | ---------------------------------------------- | ------------------------------------------- |\n| `\"change_track\"`                                           | `%{}`                                          | Defaults to an empty map                    |\n| `{\"change_track\", \"&1\"}`                                   | `%{\"title\" => \"El Pocito\"}`                    | The first argument                          |\n| `{\"change_track\", \"&1.title\"}`                             | `\"El Pocito\"`                                  | Something nested inside the first argument  |\n| `{\"change_track\", [\"&1.title\", \"&2.name\"]}`                | `[\"El Pocito\", \"Vicent Amigo\"]`                | A list combining elements of both arguments |\n| `{\"change_track\", %{song: \"&1.title\", artist: \"&2.name\"}}` | `%{song: \"El Pocito\", artist: \"Vicent Amigo\"}` | A map combining elements of both arguments  |\n\nMany frameworks will only have one argument so you will often be using `\"&1.xxx\"` etc.","ref":"readme.html#callback-parameters"},{"type":"extras","title":"Supported adapters - Komodo","doc":"Simple adapters for the following are provided\n\n- [Svelte](https://github.com/hungry-egg/komodo/tree/main/packages/komodo-svelte)\n- [React](https://github.com/hungry-egg/komodo/tree/main/packages/komodo-react)\n- [Vue](https://github.com/hungry-egg/komodo/tree/main/packages/komodo-vue)\n\nAdditionally, the `komodo` package provides an adapter for working with custom elements.","ref":"readme.html#supported-adapters"},{"type":"extras","title":"Custom elements adapter - Komodo","doc":"Some libraries like Angular (as well as Vue, Svelte) can compile to the web browser-standard [custom elements](https://developer.mozilla.org/en-US/docs/Web/API/Web_components). A simple adapter is provided for using these.","ref":"readme.html#custom-elements-adapter"},{"type":"extras","title":"Example - Komodo","doc":"A custom element `message-log` that takes a `messages` prop and emits a custom event (using the native `dispatchEvent`) `remove-message` could be used from liveview with\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n  <.js_component\n    id=\"msg-log\"\n    name=\"MessageLog\"\n    props={%{\n      messages: @messages\n    }}\n    callbacks={%{\n      \"remove-message\": {\"remove_message\", \"&1.detail.id\"}\n    }}\n  />\n  \"\"\"\nend\n\ndef handle_event(\"remove_message\", message_id, socket) do\n  // ...\nend\n```\n\nThis would be registered in `app.js` with:\n\n```diff\n// ...\n+ import { registerJsComponents, componentFromElement } from \"komodo\";\n\n// ...\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  // ...\n  hooks: {\n+    komodo: registerJsComponents({\n+      MessageLog: componentFromElement(\"message-log\")\n+    })\n  }\n});\n\n// ...\n```\n\nNote that depending on whether this is already done for you or not you may also need to [register the custom element](https://developer.mozilla.org/en-US/docs/Web/API/CustomElementRegistry/define) with the browser, e.g. in `app.js`:\n\n```\nwindow.customElements.define(\"message-log\", MessageLogElement);\n```","ref":"readme.html#example"},{"type":"extras","title":"Custom Adapters - Komodo","doc":"Adapters for each framework are small and easy to write for new libraries.\nEverything registered with the `registerJsComponents` hook\n\n```js\nkomodo: registerJsComponents({\n  MyComponent: MyComponent,\n});\n```\n\nlike `MyComponent` above, has the form\n\n```js\n{\n  // Called once on mount\n  mount: ( el, initialProps, callbackNames, emit) {\n    // el:\n    // the container element, a div by default\n\n    // initialProps:\n    // a {propName: value} lookup where values can anything JSON-friendly\n\n    // callbackNames:\n    // If <.js_component.../> was called with callbacks={%{cb1: ..., cb2: ...}} then this will\n    // be the array of names only, i.e. [\"cb1\", \"cb2\"]\n\n    // emit:\n    // a standard interface for callbacks, e.g. emit(\"callbackName\", arg1, arg2)\n    // In general the adapter won't change the callback name / arguments when calling this\n    // Any mapping to the parameters received by handle_event is done in the liveview\n\n    // Whatever you return from mount will be passed as the first arg to update and unmount\n    return {\n      some: \"context\"\n    }\n  },\n\n  // Called whenever props change\n  update: (context, newProps) {\n    // context:\n    // Whatever was returned from mount()\n\n    // newProps:\n    // the new props (all of them, including unchanged ones), as a {propName: value} lookup\n  }\n\n  // Called on unmount for cleaning up resources\n  unmount: (context) {\n    // context:\n    // Whatever was returned from mount()\n  }\n}\n```\n\nA Typescript type, [JsComponent](packages/komodo/src/js-component.ts), is given that can be used when working with Typescript.\n\nThe easiest way to create an adapter for another library is probably looking at one of the other adapters and doing something similar.\n\nIf creating an adapter for some framework \"Framework-X\", then the convention would be to create a factory function\n\n```ts\nconst componentFromFrameworkX: (Component: FrameworkXComponent) => JsComponent;\n```\n\nthat could be used in `app.js` like so:\n\n```diff\n// ...\n+ import { registerJsComponents } from \"komodo\";\n+ import componentFromFrameworkX from \"komodo-framework-x\";\n+ import FrameworkXSlider from \"path/to/framework-x/slider\";\n\n// ...\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  // ...\n  hooks: {\n+    komodo: registerJsComponents({\n+      Slider: componentFromFrameworkX(FrameworkXSlider)\n+    })\n  }\n});\n\n// ...\n```","ref":"readme.html#custom-adapters"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}